@@ -21,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::GPS;
 
-$VERSION = '1.37';
+$VERSION = '1.38';
 
 sub ProcessRicohText($$$);
 sub ProcessRicohRMETA($$$);
@@ -1008,7 +1008,7 @@ sub ProcessRicohRDT($$$)
     return 0 if $dirLen < 16;
     my $ee = $et->Options('ExtractEmbedded');
     unless ($ee) {
-        $et->WarnOnce('Use ExtractEmbedded option to read Ricoh real-time metadata',3);
+        $et->Warn('Use ExtractEmbedded option to read Ricoh real-time metadata',3);
         return 1;
     }
     my $endian = substr($$dataPt, 8, 2);
@@ -1017,11 +1017,11 @@ sub ProcessRicohRDT($$$)
     my $len = Get16u($dataPt, 6);
     if ($dirName eq 'RicohRDTG') {
         if ($ee < 2) {
-            $et->WarnOnce('Set ExtractEmbedded option to 2 or higher to extract frame timestamps',3);
+            $et->Warn('Set ExtractEmbedded option to 2 or higher to extract frame timestamps',3);
             return 1;
         }
         $rdtg = 0;
-        $et->WarnOnce('Unexpected RDTG record length') if $len > 8;
+        $et->Warn('Unexpected RDTG record length') if $len > 8;
     }
     if ($count * $len + 16 > $dirLen) {
         $et->Warn("Truncated $dirName data");
@@ -1078,7 +1078,7 @@ sub ProcessRicohText($$$)
             $tagInfo = {
                 Name => "Ricoh_Text_$tag",
                 Unknown => 1,
-                PrintConv => 'length($val) > 60 ? substr($val,0,55) . "[...]" : $val',
+                PrintConv => \&Image::ExifTool::LimitLongValues,
             };
             # add tag information to table
             AddTagToTable($tagTablePtr, $tag, $tagInfo);
@@ -1252,7 +1252,7 @@ interpret Ricoh maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
